#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Calculates volatility for specified instrument
	/// 
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
    /// </summary>
    [Description("Calculates volatility for specified instrument")]
    public class mtVolatility : Indicator
    {
        #region Variables
        // Wizard generated variables
        private int period = 20; // Default setting for Period
        // User defined variables (add any user defined variables below)
		private StdDev vol;
        #endregion

        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, "Value"));
            Add(new Line(Color.FromKnownColor(KnownColor.Black), 0, "Zero"));
            Overlay				= false;
        }
		
		protected override void OnStartUp()
		{
			vol = StdDev(PriceChange(),period);
		}

        protected override void OnBarUpdate()
        {
			Value[0] = vol[0];
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Value
        {
            get { return Values[0]; }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int Period
        {
            get { return period; }
            set { period = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private mtVolatility[] cachemtVolatility = null;

        private static mtVolatility checkmtVolatility = new mtVolatility();

        /// <summary>
        /// Calculates volatility for specified instrument
        /// </summary>
        /// <returns></returns>
        public mtVolatility mtVolatility(int period)
        {
            return mtVolatility(Input, period);
        }

        /// <summary>
        /// Calculates volatility for specified instrument
        /// </summary>
        /// <returns></returns>
        public mtVolatility mtVolatility(Data.IDataSeries input, int period)
        {
            if (cachemtVolatility != null)
                for (int idx = 0; idx < cachemtVolatility.Length; idx++)
                    if (cachemtVolatility[idx].Period == period && cachemtVolatility[idx].EqualsInput(input))
                        return cachemtVolatility[idx];

            lock (checkmtVolatility)
            {
                checkmtVolatility.Period = period;
                period = checkmtVolatility.Period;

                if (cachemtVolatility != null)
                    for (int idx = 0; idx < cachemtVolatility.Length; idx++)
                        if (cachemtVolatility[idx].Period == period && cachemtVolatility[idx].EqualsInput(input))
                            return cachemtVolatility[idx];

                mtVolatility indicator = new mtVolatility();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Period = period;
                Indicators.Add(indicator);
                indicator.SetUp();

                mtVolatility[] tmp = new mtVolatility[cachemtVolatility == null ? 1 : cachemtVolatility.Length + 1];
                if (cachemtVolatility != null)
                    cachemtVolatility.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cachemtVolatility = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Calculates volatility for specified instrument
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.mtVolatility mtVolatility(int period)
        {
            return _indicator.mtVolatility(Input, period);
        }

        /// <summary>
        /// Calculates volatility for specified instrument
        /// </summary>
        /// <returns></returns>
        public Indicator.mtVolatility mtVolatility(Data.IDataSeries input, int period)
        {
            return _indicator.mtVolatility(input, period);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Calculates volatility for specified instrument
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.mtVolatility mtVolatility(int period)
        {
            return _indicator.mtVolatility(Input, period);
        }

        /// <summary>
        /// Calculates volatility for specified instrument
        /// </summary>
        /// <returns></returns>
        public Indicator.mtVolatility mtVolatility(Data.IDataSeries input, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.mtVolatility(input, period);
        }
    }
}
#endregion
